'''
Copyright (c) 2014 Pivotal Software, Inc.  All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
Unless required by applicable law or agreed to in writing, software distributed under the License 
is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
See the License for the specific language governing permissions and limitations under the License.
'''


from datetime import datetime
from Repository import *


class GemfireClient:
    '''
    Allows application to store and access data stored in GemFire over REST. Please see:
    http://gemfire.docs.pivotal.io/latest/userguide/index.html#gemfire_rest/setup_config.html
    for instructions on setting up GemFire's REST service.
    '''

    def __init__(self, hostname, port, debug_mode=False):
        '''Initializes the Client with the given hostname and port'''
        self.hostname = hostname
        self.port = port
        self.base_url = "http://" + hostname + ":" + str(port) + "/gemfire-api/v1/"
        if debug_mode:
            logging.basicConfig(filename=(datetime.now().strftime('pyrest_%H_%M_%d_%m_%Y.log')), level=logging.DEBUG,
                                format=(
                                    '%(filename)s: ''%(levelname)s: ''%(funcName)s(): ''%(lineno)d:\t''%(message)s'))
            logging.info('Started Client')
        self.connection()

    def connection(self):
        ''' Checks connection to the server '''
        data = requests.get(self.base_url)
        if data.status_code == 200:
            logging.info("Client successfully connected to server at " + self.hostname)
            return True
        else:
            self.error_response(data)

    def list_all_regions(self):
        ''' Lists all names of Regions present in the server '''
        data = requests.get(self.base_url)
        logging.debug("Sending request to " + self.base_url)
        fdata = jsonpickle.decode(data.text)
        rnames = fdata['regions']
        names = [region['name'] for region in rnames]
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return names
        else:
            self.error_response(data)

    def create_repository(self, name):
        ''' Initializes and returns a Repository Object '''
        data = requests.get(self.base_url).json()
        logging.debug("Sending request to " + self.base_url)
        rnames = data['regions']
        names = [region['name'] for region in rnames]
        for n in names:
            if n == name:
                logging.debug("Returned back Repository object for " + name)
                type = rnames[names.index(name)]["type"]
                return Repository(name, self.base_url, type)
        else:
            logging.debug("Repository " + name + " does not exist in the server")
            print "Repository " + name + " does not exist in the server"
            return False

    def list_all_queries(self):
        ''' Lists all stored Queries in the server '''
        data = requests.get(self.base_url + "/queries")
        logging.debug("Sending request to " + self.base_url + "/queries")
        fdata = jsonpickle.decode(data.text)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return fdata["queries"]
        else:
            self.error_response(data)

    def run_query(self, query_id, query_args):
        ''' Runs the Query with specified parameters '''
        args = "{" + "'args:'" + " [" + str(query_args) + "]}"
        url = self.base_url + "queries/" + query_id
        headers = {'content-type': 'application/json'}
        jvalue = jsonpickle.encode(args)
        data = requests.post(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return jsonpickle.decode(data.text)
        else:
            self.error_response(data)

    def new_query(self, query_id, query_string):
        ''' Creates a new Query and adds it to the server '''
        url = self.base_url + "/queries?id=" + str(query_id) + "&q=" + str(query_string)
        headers = {'content-type': 'application/json'}
        jvalue = jsonpickle.encode(query_string)
        data = requests.post(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 201:
            logging.debug("Query " + query_id + " was successfully added to the server")
            return True
        else:
            self.error_response(data)

    def adhoc_query(self, query_string):
        ''' Runs an adhoc Query '''
        url = self.base_url + "queries/adhoc?q=" + str(query_string)
        data = requests.get(url)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return jsonpickle.decode(data.text)
        else:
            self.error_response(data)

    def list_all_functions(self):
        ''' List all stored function ID's stored on server '''
        url = self.base_url + "functions"
        data = requests.get(url)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return jsonpickle.decode(data.text)
        else:
            self.error_response(data)

    def execute_function(self, region_name, func_id, value):
        ''' Run function '''
        url = self.base_url + "functions/" + str(func_id) + "?onRegion=" + str(region_name)
        headers = {'content-type': 'application/json'}
        jvalue = jsonpickle.encode(value)
        data = requests.post(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return jsonpickle.decode(data.text)
        else:
            self.error_response(data)

    def error_response(self, data):
        ''' Processes HTTP error responses '''
        if data != 400 or data != 409 or data != 405:
            logging.warning("Response from server: " + str(data.status_code) + " " + data.reason + " - " + data.text)
            print str(data.status_code) + ": " + data.reason
            return False
        else:
            logging.debug("Response from server: " + str(data.status_code) + " " + data.reason + " - " + data.text)
            return False



'''
Copyright (c) 2014 Pivotal Software, Inc.  All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
Unless required by applicable law or agreed to in writing, software distributed under the License 
is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
See the License for the specific language governing permissions and limitations under the License.
'''


from Region import *


class Repository:

    def __init__(self, name, base_url, type):
        ''' Initializes a Repository '''
        self.name = name
        self.base_url = base_url + name
        self.type = type
        self.region = self.get_region()

    def delete(self, entities):
        ''' Deletes the specified Object from the region '''
        if isinstance(entities, list):
            temp = ' , '.join(str(key.id) for key in entities)
            return self.region.delete(temp)
        if isinstance(entities, int):
            return self.region.delete(entities)
        else:
            return self.region.delete(entities.id)

    def delete_all(self):
        ''' Deletes all data in the region '''
        return self.region.clear()

    def exists(self, id):
        ''' Checks whether an Object with the given id exists '''
        value = self.region.get(id)
        if value is None:
            return False
        else:
            return True

    def find_all(self):
        ''' Returns all data in the region '''
        return self.region.get_all()

    def find(self, ids):
        ''' Retrieves Object(s) by the given ID(s) from the region '''
        if isinstance(ids, list):
            temp = ",".join(str(key) for key in ids)
            return self.region.get(temp)
        else:
            return self.region.get(ids)

    def save(self, entities):
        ''' Saves(s) all given Objects in the region '''
        if isinstance(entities, list):
            item = {}
            for entity in entities:
                item[entity.id] = entity
            return self.region.put_all(item)
        else:
            return self.region.put(entities.id, entities)

    def get_region(self):
        ''' Instantiates and returns a Region object '''
        return Region(self.name, self.base_url, self.type)



'''
Copyright (c) 2014 Pivotal Software, Inc.  All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the "License"); 
you may not use this file except in compliance with the License. 
You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
Unless required by applicable law or agreed to in writing, software distributed under the License 
is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
See the License for the specific language governing permissions and limitations under the License.
'''


import logging
import requests
import jsonpickle


class Region:

    def __init__(self, name, base_url, type):
        ''' Initializes a Region '''
        self.name = name
        self.base_url = base_url
        self.type = type

    def get_all(self):
        ''' Returns all the data in a Region '''
        url = self.base_url + "?ALL"
        data = requests.get(url)
        logging.debug("Sending request to " + url)
        fdata = jsonpickle.decode(data.text)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return fdata[self.name]
        else:
            self.error_response(data)

    def create(self, key, value):
        ''' Creates a new data value in the Region if the key is absent '''
        url = self.base_url + "?key=" + str(key)
        headers = {'content-type': 'application/json'}
        jvalue = jsonpickle.encode(value)
        data = requests.post(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 201:
            logging.debug("The value " + str(value) + " was created in the region for the key " + str(key))
            return True
        else:
            self.error_response(data)

    def put(self, key, value):
        ''' Updates or inserts data for a specified key '''
        url = self.base_url + "/" + str(key)
        headers = {'content-type': 'application/json'}
        jvalue = jsonpickle.encode(value)
        data = requests.put(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("The value " + str(value) + " was put in the region for the key " + str(key))
            return True
        else:
            self.error_response(data)

    def keys(self):
        ''' Returns all keys in the Region '''
        url = self.base_url + "/keys"
        data = requests.get(url)
        logging.debug("Sending request to " + url)
        fdata = jsonpickle.decode(data.text)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return fdata["keys"]
        else:
            self.error_response(data)

    def get(self, *arg):
        ''' Returns the data value for a specified key '''
        sub_url = ','.join(str(key) for key in arg)
        url = self.base_url + "/" + sub_url + "?ignoreMissingKey=true"
        data = requests.get(url)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return jsonpickle.decode(data.text)
        else:
            self.error_response(data)

    def __getitem__(self, key):
        ''' Method to support region[key] notion '''
        url = self.base_url + "/" + str(key) + "?ignoreMissingKey=true"
        data = requests.get(url)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("Response from server: " + " ,".join(data))
            return jsonpickle.decode(data.text)
        else:
            self.error_response(data)

    def put_all(self, item):
        ''' Insert or updates data for multiple keys specified by a hashtable '''
        sub_url = ','.join(str(keys) for keys in item)
        url = self.base_url + "/" + sub_url
        headers = {'content-type': 'application/json'}
        jvalue = jsonpickle.encode(item.values())
        data = requests.put(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug(str(item) + " was put into the region")
            return True
        else:
            self.error_response(data)

    def update(self, key, value):
        ''' Updates the data in a region only if the specified key is present '''
        url = self.base_url + "/" + str(key) + "?op=REPLACE"
        headers = {'content-type': 'application/json'}
        jvalue = jsonpickle.encode(value)
        data = requests.put(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("The value at key: " + str(key) + " was updated to " + str(value))
            return True
        else:
            self.error_response(data)

    def compare_and_set(self, key, oldvalue, newvalue):
        ''' Compares old values and if identical replaces with a new value '''
        url = self.base_url + "/" + str(key) + "?op=CAS"
        headers = {'content-type': 'application/json'}
        value = {"@old": oldvalue, "@new": newvalue}
        jvalue = jsonpickle.encode(value)
        data = requests.put(url, data=jvalue, headers=headers)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug(str(oldvalue) + " was replaced with " + str(newvalue) + " at the key " + str(key))
            return True
        else:
            self.error_response(data)

    def delete(self, *arg):
        ''' Deletes the corresponding data value for the specified key '''
        sub_url = ','.join(str(key) for key in arg)
        url = self.base_url + "/" + sub_url
        data = requests.delete(url)
        logging.debug("Sending request to " + url)
        if data.status_code == 200:
            logging.debug("The values for the keys: " + str(arg) + " were deleted from the region")
            return True
        else:
            self.error_response(data)

    def clear(self):
        ''' Deletes all data in the Region '''
        if self.type == "REPLICATE":
            data = requests.delete(self.base_url)
            if data.status_code == 200:
                logging.debug("All data was cleared from the region")
                return True
            else:
                self.error_response(data)
        if self.type == "PARTITION":
            keys = self.keys()
            temp = ",".join(str(key) for key in keys)
            self.delete(temp)
            return True

    def error_response(self, data):
        ''' Processes HTTP error responses '''
        if data != 400 or data != 409 or data != 405:
            logging.warning("Response from server: " + str(data.status_code) + " " + data.reason + " - " + data.text)
            print str(data.status_code) + ": " + data.reason
            return False
        else:
            logging.debug("Response from server: " + str(data.status_code) + " " + data.reason + " - " + data.text)
            return False




'''
Copyright (c) 2014 Pivotal Software, Inc.  All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
Unless required by applicable law or agreed to in writing, software distributed under the License
is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and limitations under the License.
'''

from distutils.core import setup

setup(
    name='gemfire-rest',
    version='1.0.1',
    description='Python client for GemFire\'s REST service ',
    long_description=open('README.txt').read(),
    url='https://github.com/gemfire/py-gemfire-rest',
    packages=['gemfire-client',],
    license='Apache 2.0',
    keywords=['GemFire', 'key-value store'],
    install_requires=['logging', 'requests', 'jsonpickle'],
    author='Madhura Kulkarni, Anirudh Jain',
    author_email='madhurak25@gmail.com',
    maintainer='Swapnil Bawaskar',
    maintainer_email='sbawaskar@pivotal.io',
)
